"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const operators_1 = require("./operators");
const errors_1 = require("./errors");
const conditions_1 = require("./conditions");
const lodash_get_1 = require("lodash.get");
exports.findInConditions = ({ condition, target, flag }) => {
    for (const key in operators_1.conditions) {
        if (operators_1.conditions[key].alias.includes(condition)) {
            return operators_1.conditions[key][target];
        }
    }
    throw (new errors_1.CheckError({
        'status': false,
        'operator': condition,
        'target': target,
        'flag': flag,
        'reason': `Error: operator: "${condition}" does not exist or doesn't have "${target}" attribute`,
    }));
};
exports.buildArg = (key) => {
    const tmp = key.substring(key.indexOf(operators_1.SEP_OPERATOR), key.length);
    let operator;
    let path;
    let flag = [];
    if (tmp === key) {
        operator = operators_1.DEFAULT_CONDITION;
        path = key;
    }
    else {
        flag = tmp.split(operators_1.SEP_FLAG);
        operator = flag[2];
        flag.splice(0, 3);
        path = key.substring(0, key.indexOf(operators_1.SEP_OPERATOR));
    }
    while (path.includes(operators_1.SEP_PATH))
        path = path.replace(operators_1.SEP_PATH, operators_1.GET_SEPARATOR);
    return { operator, path, flag };
};
exports.handleComplexRequest = ({ operator, path, flag, context, value }) => {
    let result = [];
    let match = [];
    let tab = path.split('.');
    const tocheck = tab.pop();
    const expectedTab = lodash_get_1.get(context, tab.join('.'));
    const call = exports.findInConditions({ condition: flag[0], target: 'call', flag: flag });
    expectedTab.forEach((expected) => {
        if (typeof expected === 'undefined' || typeof value === 'undefined') {
            return {
                'status': false,
                'operator': operator,
                'given_value': value,
                'expected': expected,
                'flag': flag,
                'reason': `${status ? 'Success' : 'Fail'} because ${operator} of "${expected}" is ${status ? '' : 'not'} ${exports.findInConditions({
                    condition: flag[0],
                    target: 'humanlyReadableAs',
                    flag: flag
                })} "${value}"`,
            };
        }
        match.push({ 'value': value, 'expected': expected[tocheck] });
        result.push(call(value, expected[tocheck], flag));
    });
    const status = exports.findInConditions({ condition: operator, target: 'call', flag: flag })(result);
    return {
        'status': status,
        'operator': operator,
        'given_value': value,
        'expected': match,
        'flag': flag,
        'reason': `${status ? 'Success' : 'Fail'} because "${match}" is ${status ? '' : 'not'} ${exports.findInConditions({
            condition: operator,
            target: 'humanlyReadableAs',
            flag: flag
        })} "${value}"`,
    };
};
exports.check = ({ context, key, value, options = conditions_1.defaultOptions }) => {
    let { operator, path, flag } = exports.buildArg(key);
    if (operator === operators_1.EVERY_STRING || operator === operators_1.SOME_STRING || operator === operators_1.NONE_STRING) {
        return exports.handleComplexRequest({ operator: operator, path: path, flag: flag, context: context, value: value });
    }
    const expectedValue = lodash_get_1.get(context, path);
    if (typeof expectedValue === 'undefined') {
        if (options.strictMatch) {
            return {
                'status': false,
                'operator': operator,
                'given_value': value,
                'expected': expectedValue,
                'flag': flag,
                'reason': `'Fail because "${expectedValue}" is not ${exports.findInConditions({ condition: operator, target: 'humanlyReadableAs', flag: flag })} "${value}"`,
            };
        }
        else {
            throw (new errors_1.ValueNotFound({
                'status': null,
                'operator': operator,
                'path': path,
                'expected': expectedValue,
                'reason': `Error: path: ${path} is not defined in context`,
            }));
        }
    }
    const call = exports.findInConditions({ condition: operator, target: 'call', flag: flag });
    const status = call(value, expectedValue, flag);
    return {
        'status': status,
        'operator': operator,
        'given_value': value,
        'expected': expectedValue,
        'flag': flag,
        'reason': `${status ? 'Success' : 'Fail'} because "${expectedValue}" is ${status ? '' : 'not'} ${exports.findInConditions({
            condition: operator,
            target: 'humanlyReadableAs',
            flag: flag
        })} "${value}"`,
    };
};
