"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const check_1 = require("./check");
const operators_1 = require("./operators");
const operators = {
    'AND': operators_1.and,
    'OR': operators_1.or,
    'NOT': operators_1.not,
};
exports.defaultOptions = {
    'strictMatch': false,
};
const checkContextMatchesConditions = ({ filters, context, options = exports.defaultOptions }) => {
    let returnValues = {};
    let ignoredConditionsArr = [];
    filters = exports.formatFilters(Object.assign({}, filters));
    for (let [key, value] of Object.entries(filters)) {
        if (key in operators) {
            value.forEach((condition) => {
                const { status, ignoredConditions } = checkContextMatchesConditions({ filters: condition, context: context, options: options });
                if (status === false && ignoredConditions !== null) {
                    ignoredConditionsArr.push(ignoredConditions);
                }
                else {
                    returnValues[key] = returnValues[key] || [];
                    returnValues[key].push(status);
                }
            });
        }
        else {
            try {
                const result = check_1.check({ context: context, key: key, value: value, options: options });
                filters[key + '__result'] = result;
                if (result['status'] === false) {
                    return {
                        'status': result['status'],
                        'reason': result['reason'],
                        'ignoredConditions': ignoredConditionsArr.length > 0 ? ignoredConditionsArr : null,
                    };
                }
            }
            catch (e) {
                if (e.name === 'CheckError') {
                    return { 'status': false, 'ignoredConditions': e.data };
                }
                if (e.name === 'ValueNotFound') {
                    return { 'status': false, 'ignoredConditions': e.data };
                }
                else {
                    throw e;
                }
            }
        }
    }
    if (Object.entries(returnValues).length === 0 && returnValues.constructor === Object) {
        return { 'status': true, 'ignoredConditions': ignoredConditionsArr.length > 0 ? ignoredConditionsArr : null };
    }
    let operatorsValues = [];
    for (let [key, value] of Object.entries(returnValues)) {
        operatorsValues.push(operators[key](value));
    }
    const returnStatus = operators_1.and(operatorsValues);
    return {
        'status': returnStatus,
        'ignoredConditions': ignoredConditionsArr.length > 0 ? ignoredConditionsArr : null,
        'reason': !returnStatus ? 'Top-level "AND" condition returned false' : null,
    };
};
exports.formatFilters = (filters) => {
    for (let [key, value] of Object.entries(filters)) {
        if (key in operators) {
            for (let i = 0; i < value.length; i++) {
                if (Object.keys(value[i]).length > 1) {
                    const ruleObjectBackup = value[i];
                    filters[key].splice(i - 1, 1);
                    for (let [rule, expected] of Object.entries(ruleObjectBackup)) {
                        let newObj = {};
                        newObj[rule] = expected;
                        filters[key].push(newObj);
                    }
                }
            }
        }
    }
    return filters;
};
exports.default = checkContextMatchesConditions;
